% compute dimension of identifiability for Example 2.7
% original model is identifiable
% with initial condition has 1 dimension of unidentifiability

% parameters
syms p12 p13 p21
p = [p12 p13 p21];

% upper bound
for r = 0:5
    % setup functions and variables
    syms t;
    varX = [];
    varU = [];
    varY = [];
    X = zeros(3,1)*p12;
    Y = 0*p12;
    U = 0*p12;
    for j = 0:r+1
        for k = 1:3
            eval(['syms x',num2str(k),num2str(j)]);
            X(k) = X(k) + eval(['x',num2str(k),num2str(j)])*t^j/factorial(j);
            varX = [varX eval(['x',num2str(k),num2str(j)])];
        end;

        if j <= r
            eval(['syms y',num2str(j)]);
            Y = Y + eval(['y',num2str(j)])*t^j/factorial(j);
            varY = [varY eval(['y',num2str(j)])];

            eval(['syms u',num2str(j)]);
            U = U + eval(['u',num2str(j)])*t^j/factorial(j);
            varU = [varU eval(['u',num2str(j)])];
        end;
    end;
    Xd = diff(X,t);

    G1 = p13*X(3) + p12*X(2) - p21*X(1) + U - Xd(1);
    G2 = p21*X(1) - p12*X(2) - Xd(2);
    G3 = -p13*X(3) - Xd(3);
    G4 = X(2) - Y;
    G = [G1;G2;G3;G4];

    % compute coefficients of 1,t,t^2,..,t^r
    F = [];
    for j = 0:r
        F = [F;subs(diff(G,t,j),t,0)];
    end;

    for UseRestriction = 0:1
        % compute a solution
        x0 = [x10 x20 x30];
        x0e = rand(size(x0))*2-1;

        if UseRestriction == 1
            x0e(3) = 0;
        end;

        pe = rand(size(p))*2-1;
        varUe = rand(size(varU))*2-1;
        Fe = subs(F,[x0 p varU],[x0e pe varUe]);

        % compute unique solution to linear system
        S = solve(Fe);

        if UseRestriction == 1
            F = [x30;F];
        end;

        varX1 = varX(4:end);
        varX1e = zeros(1,length(varX1));
        for j = 1:length(varX1)
            varX1e(j) = double(eval(['S.',char(varX1(j))]));
        end;
        varYe = zeros(size(varY));
        for j = 1:length(varY)
            varYe(j) = double(eval(['S.',char(varY(j))]));
        end;

        % compute rank of Jacobian w.r.t. p, x
        J = jacobian(F,[p varX]);
        Je = double(subs(J,[x0 varX1 p varU varY],[x0e varX1e pe varUe varYe]));
        rPX = double(vpa(rank(Je)));
        nPX = double(size(null(Je),2));
        cPX = size(J,2) - rPX;

        % compute rank of Jacobian w.r.t. x
        J = jacobian(F,[varX]);
        Je = double(subs(J,[x0 varX1 p varU varY],[x0e varX1e pe varUe varYe]));
        rX = double(vpa(rank(Je)));
        nX = double(size(null(Je),2));
        cX = size(J,2) - rX;

        disp(' ');
        if UseRestriction == 1
            disp('With Initial Condition:');
        else
            disp('Original Model:');
        end;
        disp(['r = ',num2str(r)]);
        disp(['corank of p,x = ',num2str(cPX)]);
        disp(['corank of x = ',num2str(cX)]);

        disp(['null of p,x = ',num2str(nPX)]);
        disp(['null of x = ',num2str(nX)]);
        disp(' ');
    end;
end;
