
load 'SamplePoints.mat'; % loads the sample points into 'Pts'
OrigPt = [-0.669-0.1758i          -0.1669+0.3165i           2.3433+0.6225i          -0.6286-0.1868i           0.4005-0.5144i           2.1248-0.6011i           1.1295-0.8604i           -0.421+0.6785i          -1.1126-0.0416i           -0.688+0.3317i];

% compute dimensions of unidentifiability
syms a11 a12 a21 a22 a23 a33 a34 a42 a43 a44;
A = [a11 a12 a21 a22 a23 a33 a34 a42 a43 a44];

c1 = a11*a22*a33*a44 - a11*a22*a34*a43 + a11*a23*a34*a42 - a12*a21*a33*a44 + a12*a21*a34*a43;
c2 = a12*a21*a33 - a11*a22*a33 - a11*a22*a44 + a12*a21*a44 - a11*a33*a44 + a11*a34*a43 - a22*a33*a44 + a22*a34*a43 - a23*a34*a42;
c3 = a11*a22 - a12*a21 + a11*a33 + a11*a44 + a22*a33 + a22*a44 + a33*a44 - a34*a43;
c4 = - a11 - a22 - a33 - a44;
c5 = a22*a33*a44 - a22*a34*a43 + a23*a34*a42;
c6 = a34*a43 - a22*a44 - a33*a44 - a22*a33;
c7 = a22 + a33 + a44;

C = [c1 c2 c3 c4 c5 c6 c7];
J = jacobian(C,A);
disp(['dimensions of unidentifiability: ',num2str(double(size(J,2)-rank(J)))]);
disp(' ');

% the exactness recovery found 3 linear and 3 quadratic polynomials.  Show that the third quadratic polynomial is not indenpendent 
f1 = a11;
f2 = a22;
f3 = a33+a44;
f4 = a12*a21;
f5 = a33*a44-a34*a43;
g = a33^2 + a44^2 + 2*a34*a43;

C2 = [f1 f2 f3 f4 f5 g];
JC = jacobian(C2,A);

disp(['rank(f1,..,f5) = ',num2str(double(rank(JC(1:5,:))))]);
disp(['rank(f1,..,f5,g) = ',num2str(double(rank(JC)))]);
disp(['This shows that g = a33^2 + a44^2 + 2*a34*a43']);
disp(['  is not algebraically independent']);
disp(' ');

% One cubic polynomial
f6 = a23*a34*a42;

C3 = [f1 f2 f3 f4 f5 f6];

JC = jacobian(C3);
disp(['rank(f1,..,f6) = ',num2str(double(rank(JC)))]);
disp(' ');

P1 = -C(4)-C(7);
P2 = (C(6)^2 + C(6)*C(7)^2 + C(4)*C(6)*C(7) + C(3)*C(6) + C(5)*C(7) + C(1) + C(4)*C(5))/(C(2) + C(5) + C(4)*C(6) + C(6)*C(7));
P3 = -(C(2) + C(5) + C(4)*C(6) + C(6)*C(7))/(C(7)^2 + C(4)*C(7) + C(3) + C(6));
P4 = - C(7)^2 - C(4)*C(7) - C(3) - C(6);
P5 = (C(1) + C(4)*C(5) + C(5)*C(7))/(C(7)^2 + C(4)*C(7) + C(3) + C(6));
P6 = -(C(1)^2 + 2*C(1)*C(4)*C(5) + C(1)*C(4)*C(6)*C(7) + 2*C(1)*C(5)*C(7) + C(1)*C(6)^2 + C(1)*C(6)*C(7)^2 + C(3)*C(1)*C(6) + C(4)^2*C(5)^2 + C(4)*C(5)^2*C(7) - C(2)*C(4)*C(5)*C(7) - C(5)^2*C(6) - C(3)*C(5)^2 - C(2)*C(5)*C(6) - C(2)*C(5)*C(7)^2 - C(2)*C(3)*C(5))/(C(2)*C(3) + C(2)*C(6) + C(3)*C(5) + C(5)*C(6) + C(2)*C(7)^2 + C(4)*C(6)^2 + C(5)*C(7)^2 + C(6)^2*C(7) + C(6)*C(7)^3 + C(2)*C(4)*C(7) + C(3)*C(4)*C(6) + C(3)*C(6)*C(7) + C(4)*C(5)*C(7) + 2*C(4)*C(6)*C(7)^2 + C(4)^2*C(6)*C(7));

disp(['simplify(f1 - P1): ',char(simplify(C3(1)-P1))]);
disp(['simplify(f2 - P2): ',char(simplify(C3(2)-P2))]);
disp(['simplify(f3 - P3): ',char(simplify(C3(3)-P3))]);
disp(['simplify(f4 - P4): ',char(simplify(C3(4)-P4))]);
disp(['simplify(f5 - P5): ',char(simplify(C3(5)-P5))]);
disp(['simplify(f6 - P6): ',char(simplify(C3(6)-P6))]);
disp(' ');


